import type { Rule } from 'eslint';
import type { Identifier, SimpleLiteral } from 'estree';

interface ImportAttribute {
  key: Identifier | SimpleLiteral;
  value: SimpleLiteral;
}

const requireImportAttribute: Rule.RuleModule = {
  meta: {
    type: 'problem',
    docs: {
      description:
        'Requires JSON imports to have a "with { type: \'json\' }" import attribute.',
      category: 'Possible Errors',
      recommended: false,
    },
    fixable: 'code',
    schema: [],
    messages: {
      missingJsonTypeAttribute:
        'JSON imports must have a "with { type: \'json\' }" import attribute.',
    },
  },
  create(context) {
    return {
      ImportDeclaration(node) {
        // Only check modules that end with ".json"
        if (
          typeof node.source.value === 'string' &&
          node.source.value.endsWith('.json')
        ) {
          // Cast the node to our extended interface to access the assertions.
          const importNode = node;
          const assertions = (
            importNode as {
              assertions?: ImportAttribute[];
            }
          ).assertions;

          const hasJsonType = assertions?.some(attr => {
            const keyName =
              (attr.key as Identifier).name ?? (attr.key as SimpleLiteral).value;
            return keyName === 'type' && attr.value.value === 'json';
          });

          if (!hasJsonType) {
            context.report({
              node,
              messageId: 'missingJsonTypeAttribute',
              fix: fixer =>
                fixer.insertTextAfterRange(node.source.range!, ' with { type: "json" }'),
            });
          }
        }
      },
    };
  },
};

export default requireImportAttribute;
